# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'listobs',
              1: 'A priori flagging',
              2: 'Generation and time averaging of the WVR cal table',
              3: 'Generation of the Tsys cal table',
              4: 'Generation of the antenna position cal table',
              5: 'Application of the WVR, Tsys and antpos cal tables',
              6: 'Split out science SPWs and time average',
              7: 'Listobs, clear pointing table, and save original flags',
              8: 'Initial flagging',
              9: 'Putting a model for the flux calibrator(s)',
              10: 'Save flags before bandpass cal',
              11: 'Bandpass calibration',
              12: 'Save flags before gain cal',
              13: 'Gain calibration',
              14: 'Save flags before applycal cal',
              15: 'Application of the bandpass and gain cal tables'}

# mysteps = range(0,7)
# mysteps = range(6,16)

try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

es = aU.stuffForScienceDataReduction() # NotForPI


# Using reference antenna = DV09

print "# A priori calibration"

# listobs
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  listobs(vis = 'uid___A002_X3b55d6_X875.ms',
    listfile = 'uid___A002_X3b55d6_X875.ms.listobs')
  
  

# A priori flagging
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X3b55d6_X875.ms',
    mode = 'manualflag',
    autocorr = T,
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_X3b55d6_X875.ms',
    mode = 'manualflag',
    intent = '*POINTING*,*SIDEBAND_RATIO*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_X3b55d6_X875.ms',
    flagmode = 'table',
    optype = 'plot')
  
  flagcmd(vis = 'uid___A002_X3b55d6_X875.ms',
    flagmode = 'table',
    optype = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system("wvrgcal --ms uid___A002_X3b55d6_X875.ms --output uid___A002_X3b55d6_X875.ms.wvr --toffset -1 --statsource 'Abell 1835' --segsource --tie '2,3' | tee uid___A002_X3b55d6_X875.ms.wvrgcal")
  
  smoothcal(vis = 'uid___A002_X3b55d6_X875.ms',
    tablein = 'uid___A002_X3b55d6_X875.ms.wvr',
    caltable = 'uid___A002_X3b55d6_X875.ms.wvr.smooth',
    smoothtype = 'mean',
    smoothtime = 6.048)
  

# Generation of the Tsys cal table
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  gencal(vis = 'uid___A002_X3b55d6_X875.ms',
    caltable = 'uid___A002_X3b55d6_X875.ms.tsys',
    caltype = 'tsys')
  
  interTsys = aU.InterpolateTsys('uid___A002_X3b55d6_X875.ms.tsys')
  interTsys.correctBadTimes(force=True)
  interTsys.assignFieldAndScanToSolution()
  interTsys.getTdmFdmSpw()
  interTsys.interpolateTsys()
  
  clearstat()
  
  es.checkCalTable('uid___A002_X3b55d6_X875.ms.tsys.fdm', interactive=False) # NotForPI
  

# Generation of the antenna position cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Warning: no baseline run found for antenna DV03.
  
  gencal(vis = 'uid___A002_X3b55d6_X875.ms',
    caltable = 'uid___A002_X3b55d6_X875.ms.antpos',
    caltype = 'antpos',
    antenna = 'PM01,DV18,DV09,DV11,DV10,DV13,DV12,DA41,DA43,DV16,DV07,DA44,DV15',
    parameter = [0.000160260512791,2.02531971992e-05,-0.000227201179669,0.000185548765409,-0.000295159421836,0.000527990124298,-9.13155434405e-05,-0.00020730757001,-2.07226473519e-07,7.30585306883e-05,-0.000288729555905,-0.000365847256035,0.000563478778251,-0.000220686599552,-7.53531103418e-05,0.000548524243538,-0.000540671697393,-1.4569484857e-05,0.000188886938791,8.49811240384e-05,-0.000112044334579,0.00060776527971,-0.000525096431375,-0.000386487226933,0.000553364867527,-0.000240854266319,-2.7341283502e-05,-8.58749711454e-05,0.000326200341151,7.17413880783e-05,-0.000127295594186,-7.17936973442e-05,2.26825201944e-05,0.000429610948498,-0.000416420822515,-0.000216206224488,-0.000457407906651,0.000259945169091,-0.000202616676688])
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  applycal(vis = 'uid___A002_X3b55d6_X875.ms',
    field = '0',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X3b55d6_X875.ms.tsys.fdm', 'uid___A002_X3b55d6_X875.ms.wvr.smooth', 'uid___A002_X3b55d6_X875.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear',
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X3b55d6_X875.ms',
    field = '1',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X3b55d6_X875.ms.tsys.fdm', 'uid___A002_X3b55d6_X875.ms.wvr.smooth', 'uid___A002_X3b55d6_X875.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = 'linear',
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X3b55d6_X875.ms',
    field = '2',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X3b55d6_X875.ms.tsys.fdm', 'uid___A002_X3b55d6_X875.ms.wvr.smooth', 'uid___A002_X3b55d6_X875.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear',
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X3b55d6_X875.ms',
    field = '3',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X3b55d6_X875.ms.tsys.fdm', 'uid___A002_X3b55d6_X875.ms.wvr.smooth', 'uid___A002_X3b55d6_X875.ms.antpos'],
    gainfield = ['3', '', ''],
    interp = 'linear',
    calwt = T,
    flagbackup = F)
  
  es.getCalWeightStats('uid___A002_X3b55d6_X875.ms') # NotForPI
  

### Correct a 4ns delay for DV05

  print "Correct a 4ns delay for DV05" 

  gencal(vis='uid___A002_X3b55d6_X875.ms',caltable='uid___A002_X3b55d6_X875.ms.del.K',caltype='sbd', spw='23',antenna='DV05',pol='X,Y', parameter=[-4.,-4.]) 

  applycal(vis='uid___A002_X3b55d6_X875.ms', flagbackup=F, spw='23',interp=['nearest'], gaintable=['uid___A002_X3b55d6_X875.ms.del.K'])





# Split out science SPWs and time average
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  split(vis = 'uid___A002_X3b55d6_X875.ms',
    outputvis = 'uid___A002_X3b55d6_X875.ms.split',
    datacolumn = 'corrected',
    spw = '17,19,21,23',
    timebin = '6.048s',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  listobs(vis = 'uid___A002_X3b55d6_X875.ms.split',
    listfile = 'uid___A002_X3b55d6_X875.ms.split.listobs')
  
  tb.open('uid___A002_X3b55d6_X875.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  flagmanager(vis = 'uid___A002_X3b55d6_X875.ms.split',
    mode = 'save',
    versionname = 'Original')
  
  

# Initial flagging
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X3b55d6_X875.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  

# Putting a model for the flux calibrator(s)
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_X3b55d6_X875.ms.split',
    field = '1', # Titan
    spw = '0,1,2,3',
    standard = 'Butler-JPL-Horizons 2010')
  
  for i in ['1']:
    plotms(vis = 'uid___A002_X3b55d6_X875.ms.split',
      xaxis = 'uvdist',
      yaxis = 'amp',
      ydatacolumn = 'model',
      field = i,
      spw = '0,1,2,3',
      avgchannel = '9999',
      coloraxis = 'spw',
      plotfile = 'uid___A002_X3b55d6_X875.ms.split.setjy.field'+i+'.png')
  

# Save flags before bandpass cal
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X3b55d6_X875.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  gaincal(vis = 'uid___A002_X3b55d6_X875.ms.split',
    caltable = 'uid___A002_X3b55d6_X875.ms.split.ap_pre_bandpass',
    field = '0', # 3C279
    spw = '0:1536~2304,1:1536~2304,2:1536~2304,3:1536~2304',
    solint = 'int',
    refant = 'DV09',
    calmode = 'ap')
  
  es.checkCalTable('uid___A002_X3b55d6_X875.ms.split.ap_pre_bandpass', interactive=False) # NotForPI
  
  bandpass(vis = 'uid___A002_X3b55d6_X875.ms.split',
    caltable = 'uid___A002_X3b55d6_X875.ms.split.bandpass',
    field = '0', # 3C279
    solint = 'inf',
    combine = 'scan',
    refant = 'DV09',
    solnorm = T,
    bandtype = 'B',
    gaintable = 'uid___A002_X3b55d6_X875.ms.split.ap_pre_bandpass')
  
  aU.smoothbandpass('uid___A002_X3b55d6_X875.ms.split.bandpass') # NotForPI
  
  es.checkCalTable('uid___A002_X3b55d6_X875.ms.split.bandpass', interactive=False) # NotForPI
  

# Save flags before gain cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X3b55d6_X875.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Note: the Solar system object used for flux calibration is highly resolved on some baselines.
  # Note: we will first determine the flux of the phase calibrator(s) on a subset of antennas.
  
  gaincal(vis = 'uid___A002_X3b55d6_X875.ms.split',
    caltable = 'uid___A002_X3b55d6_X875.ms.split.phase_short_int',
    field = '0~2', # 3C279,Titan,J1332+0200
    selectdata = T,
    antenna = 'DA41,DV07,DV09,DV11,DV12,DV16,DV17,DV18,PM01&',
    solint = 'int',
    refant = 'DV09',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X3b55d6_X875.ms.split.bandpass_smooth20flat_ri')
  
  es.checkCalTable('uid___A002_X3b55d6_X875.ms.split.phase_short_int', interactive=False) # NotForPI
  
  gaincal(vis = 'uid___A002_X3b55d6_X875.ms.split',
    caltable = 'uid___A002_X3b55d6_X875.ms.split.ampli_short_inf',
    field = '0~2', # 3C279,Titan,J1332+0200
    selectdata = T,
    antenna = 'DA41,DV07,DV09,DV11,DV12,DV16,DV17,DV18,PM01&',
    solint = 'inf',
    refant = 'DV09',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['uid___A002_X3b55d6_X875.ms.split.bandpass_smooth20flat_ri', 'uid___A002_X3b55d6_X875.ms.split.phase_short_int'])
  
  es.checkCalTable('uid___A002_X3b55d6_X875.ms.split.ampli_short_inf', interactive=False) # NotForPI
  
  casalog.setlogfile('uid___A002_X3b55d6_X875.ms.split.fluxscale')
  
  fluxscale(vis = 'uid___A002_X3b55d6_X875.ms.split',
    caltable = 'uid___A002_X3b55d6_X875.ms.split.ampli_short_inf',
    fluxtable = 'uid___A002_X3b55d6_X875.ms.split.flux_short_inf',
    reference = '1') # Titan
  
  casalog.setlogfile('')
  
  f = open('uid___A002_X3b55d6_X875.ms.split.fluxscale')
  fc = f.readlines()
  f.close()
  
  for phaseCalName in ['J1332+0200']:
    for i in range(len(fc)):
      if fc[i].find('Flux density for '+phaseCalName) != -1 and re.search('in SpW=[0-9]+( \(ref SpW=[0-9]+\))? is: [0-9]+\.[0-9]+', fc[i]) != None:
        line = (re.search('in SpW=[0-9]+( \(ref SpW=[0-9]+\))? is: [0-9]+\.[0-9]+', fc[i])).group(0)
        spwId = (line.split('='))[1].split()[0]
        flux = float((line.split(':'))[1].split()[0])
        setjy(vis = 'uid___A002_X3b55d6_X875.ms.split',
          field = phaseCalName,
          spw = spwId,
          fluxdensity = [flux,0,0,0])
  
  gaincal(vis = 'uid___A002_X3b55d6_X875.ms.split',
    caltable = 'uid___A002_X3b55d6_X875.ms.split.phase_int',
    field = '0~2', # 3C279,Titan,J1332+0200
    solint = 'int',
    refant = 'DV09',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X3b55d6_X875.ms.split.bandpass_smooth20flat_ri')
  
  es.checkCalTable('uid___A002_X3b55d6_X875.ms.split.phase_int', interactive=False) # NotForPI
  
  gaincal(vis = 'uid___A002_X3b55d6_X875.ms.split',
    caltable = 'uid___A002_X3b55d6_X875.ms.split.flux_inf',
    field = '0~2', # 3C279,Titan,J1332+0200
    solint = 'inf',
    refant = 'DV09',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['uid___A002_X3b55d6_X875.ms.split.bandpass_smooth20flat_ri', 'uid___A002_X3b55d6_X875.ms.split.phase_int'])
  
  es.checkCalTable('uid___A002_X3b55d6_X875.ms.split.flux_inf', interactive=False) # NotForPI
  
  gaincal(vis = 'uid___A002_X3b55d6_X875.ms.split',
    caltable = 'uid___A002_X3b55d6_X875.ms.split.phase_inf',
    field = '0~2', # 3C279,Titan,J1332+0200
    solint = 'inf',
    refant = 'DV09',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X3b55d6_X875.ms.split.bandpass_smooth20flat_ri')
  
  es.checkCalTable('uid___A002_X3b55d6_X875.ms.split.phase_inf', interactive=False) # NotForPI
  

# Save flags before applycal cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X3b55d6_X875.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0', '1']: # 3C279,Titan
    applycal(vis = 'uid___A002_X3b55d6_X875.ms.split',
      field = i,
      gaintable = ['uid___A002_X3b55d6_X875.ms.split.bandpass_smooth20flat_ri', 'uid___A002_X3b55d6_X875.ms.split.phase_int', 'uid___A002_X3b55d6_X875.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear',
      calwt = F,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_X3b55d6_X875.ms.split',
    field = '2,3', # Abell 1835
    gaintable = ['uid___A002_X3b55d6_X875.ms.split.bandpass_smooth20flat_ri', 'uid___A002_X3b55d6_X875.ms.split.phase_inf', 'uid___A002_X3b55d6_X875.ms.split.flux_inf'],
    gainfield = ['', '2', '2'], # J1332+0200
    interp = 'linear',
    calwt = F,
    flagbackup = F)
  

