# ALMA Data Reduction Script

# Calibration

thesteps = []
step_title = {0: 'listobs',
              1: 'A priori flagging',
              2: 'Generation and time averaging of the WVR cal table',
              3: 'Generation of the Tsys cal table',
              4: 'Generation of the antenna position cal table',
              5: 'Application of the WVR, Tsys and antpos cal tables',
              6: 'Split out science SPWs and time average',
              7: 'Listobs, clear pointing table, and save original flags',
              8: 'Initial flagging',
              9: 'Putting a model for the flux calibrator(s)',
              10: 'Save flags before bandpass cal',
              11: 'Bandpass calibration',
              12: 'Save flags before gain cal',
              13: 'Gain calibration',
              14: 'Save flags before applycal cal',
              15: 'Application of the bandpass and gain cal tables'}

# mysteps = range(0,7)
mysteps = range(6,16)

try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

es = aU.stuffForScienceDataReduction() # NotForPI


# Using reference antenna = DV09

print "# A priori calibration"

# listobs
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  listobs(vis = 'uid___A002_X3e17df_X3fd.ms',
    listfile = 'uid___A002_X3e17df_X3fd.ms.listobs')
  
  

# A priori flagging
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X3e17df_X3fd.ms',
    mode = 'manualflag',
    autocorr = T,
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_X3e17df_X3fd.ms',
    mode = 'manualflag',
    intent = '*POINTING*,*SIDEBAND_RATIO*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_X3e17df_X3fd.ms',
    flagmode = 'table',
    optype = 'plot')
  
  flagcmd(vis = 'uid___A002_X3e17df_X3fd.ms',
    flagmode = 'table',
    optype = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system("wvrgcal --ms uid___A002_X3e17df_X3fd.ms --output uid___A002_X3e17df_X3fd.ms.wvr --toffset -1 --statsource 'Abell 1835' --segsource --tie '2,3' | tee uid___A002_X3e17df_X3fd.ms.wvrgcal")
  
  smoothcal(vis = 'uid___A002_X3e17df_X3fd.ms',
    tablein = 'uid___A002_X3e17df_X3fd.ms.wvr',
    caltable = 'uid___A002_X3e17df_X3fd.ms.wvr.smooth',
    smoothtype = 'mean',
    smoothtime = 6.048)
  

# Generation of the Tsys cal table
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  gencal(vis = 'uid___A002_X3e17df_X3fd.ms',
    caltable = 'uid___A002_X3e17df_X3fd.ms.tsys',
    caltype = 'tsys')
  
  interTsys = aU.InterpolateTsys('uid___A002_X3e17df_X3fd.ms.tsys')
  interTsys.correctBadTimes(force=True)
  interTsys.assignFieldAndScanToSolution()
  interTsys.getTdmFdmSpw()
  interTsys.interpolateTsys()
  
  clearstat()
  
  es.checkCalTable('uid___A002_X3e17df_X3fd.ms.tsys.fdm', interactive=False) # NotForPI
  

# Generation of the antenna position cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Warning: the correction for antenna DV18 is larger than 2mm.
  
  # Warning: no baseline run found for antenna DV08.
  
  # Warning: no baseline run found for antenna DA47.
  
  # Warning: no baseline run found for antenna DV03.
  
  # Warning: no baseline run found for antenna DV02.
  
  # Warning: no baseline run found for antenna PM02.
  
  gencal(vis = 'uid___A002_X3e17df_X3fd.ms',
    caltable = 'uid___A002_X3e17df_X3fd.ms.antpos',
    caltype = 'antpos',
    antenna = 'DV18,DV05,DV09,DV11,DA44,DV13,DV12,DA41,DV14,DA43,DV16,DV07,DV17,DV10,DV15',
    parameter = [0.00684754783288,-0.0188271608204,-0.00732900947332,0.00105206234994,-0.000857719516042,-0.00059565765641,0.000741523189806,-0.000843518709779,-0.000409113120338,0.000922475475818,-0.00103152170777,-0.000580252148211,0.00101529745474,-0.000852548536742,-0.000478085840522,0.00100313175518,-0.00084909900337,-0.000594194887412,0.000481227821554,-2.48519197708e-05,-0.00048066949752,0.00060776527971,-0.000525096431375,-0.000386487226933,0.000521688542419,-0.000144920371688,-0.000258341151538,0.00104740676978,-0.000899314162014,-0.000483014592547,0.00094885089927,-0.000978032222759,-0.000429696337195,0.000867350985729,-0.000990079062422,-0.00038123971161,0.000920921096116,-0.00106288533016,-0.000546355296173,0.00111751207514,-0.000891124202747,-0.000492116263895,0.000567245762795,-0.000422531738877,-0.000741700641811])
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  applycal(vis = 'uid___A002_X3e17df_X3fd.ms',
    field = '0',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X3e17df_X3fd.ms.tsys.fdm', 'uid___A002_X3e17df_X3fd.ms.wvr.smooth', 'uid___A002_X3e17df_X3fd.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear',
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X3e17df_X3fd.ms',
    field = '1',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X3e17df_X3fd.ms.tsys.fdm', 'uid___A002_X3e17df_X3fd.ms.wvr.smooth', 'uid___A002_X3e17df_X3fd.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = 'linear',
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X3e17df_X3fd.ms',
    field = '2',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X3e17df_X3fd.ms.tsys.fdm', 'uid___A002_X3e17df_X3fd.ms.wvr.smooth', 'uid___A002_X3e17df_X3fd.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear',
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X3e17df_X3fd.ms',
    field = '3',
    spw = '17,19,21,23',
    gaintable = ['uid___A002_X3e17df_X3fd.ms.tsys.fdm', 'uid___A002_X3e17df_X3fd.ms.wvr.smooth', 'uid___A002_X3e17df_X3fd.ms.antpos'],
    gainfield = ['3', '', ''],
    interp = 'linear',
    calwt = T,
    flagbackup = F)
  
  es.getCalWeightStats('uid___A002_X3e17df_X3fd.ms') # NotForPI
  

# Split out science SPWs and time average
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  split(vis = 'uid___A002_X3e17df_X3fd.ms',
    outputvis = 'uid___A002_X3e17df_X3fd.ms.split',
    datacolumn = 'corrected',
    spw = '17,19,21,23',
    timebin = '6.048s',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  listobs(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    listfile = 'uid___A002_X3e17df_X3fd.ms.split.listobs')
  
  tb.open('uid___A002_X3e17df_X3fd.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  flagmanager(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    mode = 'save',
    versionname = 'Original')
  
  

# Initial flagging
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    mode = 'manualflag', antenna = '16', spw = '0',
    flagbackup = F)
   
  flagdata(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    mode = 'manualflag', antenna = '15', 
    flagbackup = F)
   
# Putting a model for the flux calibrator(s)
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    field = '1', # Titan
    spw = '0,1,2,3',
    standard = 'Butler-JPL-Horizons 2010')
  
  for i in ['1']:
    plotms(vis = 'uid___A002_X3e17df_X3fd.ms.split',
      xaxis = 'uvdist',
      yaxis = 'amp',
      ydatacolumn = 'model',
      field = i,
      spw = '0,1,2,3',
      avgchannel = '9999',
      coloraxis = 'spw',
      plotfile = 'uid___A002_X3e17df_X3fd.ms.split.setjy.field'+i+'.png')
  

# Save flags before bandpass cal
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  

 

# Bandpass calibration
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  gaincal(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    caltable = 'uid___A002_X3e17df_X3fd.ms.split.ap_pre_bandpass',
    field = '0', # 3C279
    spw = '0:1536~2304,1:1536~2304,2:1536~2304,3:1536~2304',
    solint = 'int',
    refant = 'DV09',
    calmode = 'ap')
  
  es.checkCalTable('uid___A002_X3e17df_X3fd.ms.split.ap_pre_bandpass', interactive=False) # NotForPI
  
  bandpass(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    caltable = 'uid___A002_X3e17df_X3fd.ms.split.bandpass',
    field = '0', # 3C279
    solint = 'inf',
    combine = 'scan',
    refant = 'DV09',
    solnorm = T,
    bandtype = 'B',
    gaintable = 'uid___A002_X3e17df_X3fd.ms.split.ap_pre_bandpass')
  
  aU.smoothbandpass('uid___A002_X3e17df_X3fd.ms.split.bandpass') # NotForPI
  
  es.checkCalTable('uid___A002_X3e17df_X3fd.ms.split.bandpass', interactive=False) # NotForPI
  

# Save flags before gain cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Note: the Solar system object used for flux calibration is highly resolved on some baselines.
  # Note: we will first determine the flux of the phase calibrator(s) on a subset of antennas.
  
  gaincal(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    caltable = 'uid___A002_X3e17df_X3fd.ms.split.phase_short_int',
    field = '0~2', # 3C279,Titan,J1332+0200
    selectdata = T,
    antenna = 'DA41,DA44,DA47,DV08,DV09,DV10,DV11,DV12,DV14,DV16,DV18,PM02&',
    solint = 'int',
    refant = 'DV09',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X3e17df_X3fd.ms.split.bandpass_smooth20flat_ri')
  
  es.checkCalTable('uid___A002_X3e17df_X3fd.ms.split.phase_short_int', interactive=False) # NotForPI
  
  gaincal(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    caltable = 'uid___A002_X3e17df_X3fd.ms.split.ampli_short_inf',
    field = '0~2', # 3C279,Titan,J1332+0200
    selectdata = T,
    antenna = 'DA41,DA44,DA47,DV08,DV09,DV10,DV11,DV12,DV14,DV16,DV18,PM02&',
    solint = 'inf',
    refant = 'DV09',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['uid___A002_X3e17df_X3fd.ms.split.bandpass_smooth20flat_ri', 'uid___A002_X3e17df_X3fd.ms.split.phase_short_int'])
  
  es.checkCalTable('uid___A002_X3e17df_X3fd.ms.split.ampli_short_inf', interactive=False) # NotForPI
  
  casalog.setlogfile('uid___A002_X3e17df_X3fd.ms.split.fluxscale')
  
  fluxscale(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    caltable = 'uid___A002_X3e17df_X3fd.ms.split.ampli_short_inf',
    fluxtable = 'uid___A002_X3e17df_X3fd.ms.split.flux_short_inf',
    reference = '1') # Titan
  
  casalog.setlogfile('')
  
  f = open('uid___A002_X3e17df_X3fd.ms.split.fluxscale')
  fc = f.readlines()
  f.close()
  
  for phaseCalName in ['J1332+0200']:
    for i in range(len(fc)):
      if fc[i].find('Flux density for '+phaseCalName) != -1 and re.search('in SpW=[0-9]+( \(ref SpW=[0-9]+\))? is: [0-9]+\.[0-9]+', fc[i]) != None:
        line = (re.search('in SpW=[0-9]+( \(ref SpW=[0-9]+\))? is: [0-9]+\.[0-9]+', fc[i])).group(0)
        spwId = (line.split('='))[1].split()[0]
        flux = float((line.split(':'))[1].split()[0])
        setjy(vis = 'uid___A002_X3e17df_X3fd.ms.split',
          field = phaseCalName,
          spw = spwId,
          fluxdensity = [flux,0,0,0])
  
  gaincal(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    caltable = 'uid___A002_X3e17df_X3fd.ms.split.phase_int',
    field = '0~2', # 3C279,Titan,J1332+0200
    solint = 'int',
    refant = 'DV09',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X3e17df_X3fd.ms.split.bandpass_smooth20flat_ri')
  
  es.checkCalTable('uid___A002_X3e17df_X3fd.ms.split.phase_int', interactive=False) # NotForPI
  
  gaincal(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    caltable = 'uid___A002_X3e17df_X3fd.ms.split.flux_inf',
    field = '0~2', # 3C279,Titan,J1332+0200
    solint = 'inf',
    refant = 'DV09',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['uid___A002_X3e17df_X3fd.ms.split.bandpass_smooth20flat_ri', 'uid___A002_X3e17df_X3fd.ms.split.phase_int'])
  
  es.checkCalTable('uid___A002_X3e17df_X3fd.ms.split.flux_inf', interactive=False) # NotForPI
  
  gaincal(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    caltable = 'uid___A002_X3e17df_X3fd.ms.split.phase_inf',
    field = '0~2', # 3C279,Titan,J1332+0200
    solint = 'inf',
    refant = 'DV09',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X3e17df_X3fd.ms.split.bandpass_smooth20flat_ri')
  
  es.checkCalTable('uid___A002_X3e17df_X3fd.ms.split.phase_inf', interactive=False) # NotForPI
  

# Save flags before applycal cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagmanager(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0', '1']: # 3C279,Titan
    applycal(vis = 'uid___A002_X3e17df_X3fd.ms.split',
      field = i,
      gaintable = ['uid___A002_X3e17df_X3fd.ms.split.bandpass_smooth20flat_ri', 'uid___A002_X3e17df_X3fd.ms.split.phase_int', 'uid___A002_X3e17df_X3fd.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear',
      calwt = F,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_X3e17df_X3fd.ms.split',
    field = '2,3', # Abell 1835
    gaintable = ['uid___A002_X3e17df_X3fd.ms.split.bandpass_smooth20flat_ri', 'uid___A002_X3e17df_X3fd.ms.split.phase_inf', 'uid___A002_X3e17df_X3fd.ms.split.flux_inf'],
    gainfield = ['', '2', '2'], # J1332+0200
    interp = 'linear',
    calwt = F,
    flagbackup = F)
  

